/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 *
 */

// BasicNode.h:
//
//////////////////////////////////////////////////////////////////////

// BasicNode.h: interface for the BasicNode class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASICNODE_H__623900B8_2737_4E36_8F26_97898C5BD47D__INCLUDED_)
#define AFX_BASICNODE_H__623900B8_2737_4E36_8F26_97898C5BD47D__INCLUDED_

#include <string>
#include <list>
#include "GDefine.h"

using namespace std;

class SoapSerializer;

enum NODE_TYPE { ELEMENT_NODE=1, CHARACTER_NODE};
/**
    @class BasicNode
    @brief interface for the BasicNode class.


    @author Roshan Weerasuriya (roshan@jkcsworld.com, roshan@opensource.lk)
*/
class BasicNode
{
public:
	/**
	 * Returns the number of child elements of this element. The return value
	 *  depends on the TYPE of the node.
	 * @return The number of child elements of this element. The return value
	 *  depends on the TYPE of the node.
	 */
	virtual int getNoOfChildren() = 0;
	virtual const BasicNode* getFirstChild() const = 0;
	virtual const BasicNode* getLastChild() const = 0;
	virtual const BasicNode* getChild(int iChildPosition) const = 0;
	//virtual bool operator ==( const BasicNode &objChEle)=0;
	virtual const AxisChar* getValue() const =0;
	virtual int setValue(const AxisChar* pachValue)=0;
	virtual NODE_TYPE getNodeType() const =0;
	virtual int setURI(const AxisChar* sURI) =0;
	virtual int addChild(BasicNode* pBasicNode) =0;
	virtual int setLocalName(const AxisChar* sLocalName) =0;
	virtual int setPrefix(const AxisChar* sPrefix) =0;
	virtual int serialize(SoapSerializer& pSZ) =0;
	virtual int serialize(SoapSerializer& pSZ, list<AxisChar*>& lstTmpNameSpaceStack) =0;
	virtual ~BasicNode() {};

protected:
	NODE_TYPE m_iNodeType;
	//string m_sPrefix;
	AxisChar* m_pachValue;
};

#endif // !defined(AFX_BASICNODE_H__623900B8_2737_4E36_8F26_97898C5BD47D__INCLUDED_)
