/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 *
 */

// haracterElement.h:
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HARACTERELEMENT_H__55176899_E8B1_40ED_BA39_CB8C5590B4D2__INCLUDED_)
#define AFX_HARACTERELEMENT_H__55176899_E8B1_40ED_BA39_CB8C5590B4D2__INCLUDED_

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "BasicNode.h"
/**
    @class CharacterElement
    @brief interface for the CharacterElement class.


    @author Roshan Weerasuriya (roshan@jkcs.slt.lk, roshan@opensource.lk)
*/
class CharacterElement : public BasicNode
{
public:
	bool operator ==( const CharacterElement &objChEle);
	int setValue(const AxisChar* pachValue);
	const AxisChar* getValue() const;

	/**
	 * This method is overridden to return 0 always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return 0.
	 */
	int getNoOfChildren() {return 0;}

	/**
	 * This method is overridden to return NULL always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return NULL to indicate unsuccessfull operation.
	 */
 	const BasicNode* getFirstChild() const {return NULL;}

	/**
	 * This method is overridden to return NULL always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return NULL to indicate unsuccessfull operation.
	 */
	const BasicNode* getLastChild() const {return NULL;}

	/**
	 * This method is overridden to return NULL always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return NULL to indicate unsuccessfull operation.
	 */
	const BasicNode* getChild(int iChildPosition) const {return NULL;}

	/**
	 * This method is overridden to return fail(AXIS_FAIL) always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return fail (AXIS_FAIL) to indicate unsuccessfull operation.
	 */
	int setURI(const AxisChar* sURI) {return AXIS_FAIL;}

	/**
	 * This method is overridden to return fail(AXIS_FAIL) always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return fail (AXIS_FAIL) to indicate unsuccessfull operation.
	 */
	int addChild(BasicNode* pBasicNode) {return AXIS_FAIL;}

	/**
	 * This method is overridden to return fail(AXIS_FAIL) always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return fail (AXIS_FAIL) to indicate unsuccessfull operation.
	 */
	int setLocalName(const AxisChar* sLocalName) {return AXIS_FAIL;}

	/**
	 * This method is overridden to return fail(AXIS_FAIL) always because this 
	 *  is a CharacterElement. 
	 *  A user of a CharacterElement should not use this method.
	 * @return Always return fail (AXIS_FAIL) to indicate unsuccessfull operation.
	 */
	int setPrefix(const AxisChar* sPrefix) {return AXIS_FAIL;}

	int serialize(SoapSerializer& pSZ);
	/**
	 * This method is actualy not used in this class. This method was just
	 * included to adhere its interface.
	 */
	int serialize(SoapSerializer& pSZ, list<AxisChar*>& lstTmpNameSpaceStack) {return AXIS_SUCCESS;}
	NODE_TYPE getNodeType() const;
	CharacterElement(const AxisChar* sValue);
	CharacterElement();
	virtual ~CharacterElement();
};

#endif // !defined(AFX_HARACTERELEMENT_H__55176899_E8B1_40ED_BA39_CB8C5590B4D2__INCLUDED_)
